#!/usr/bin/python
from __future__ import print_function
import os
import re
from shutil import copyfile


# looks for pattern in a string; returns True if found, False if not
def Find(pat, text):
    match = re.search(pat, text)
    return match


data_summary_table = "fin.csv"
working_directory = os.getcwd()  # assumes that you are already in processing folder


space_group = "!SPACE_GROUP_NUMBER=  4 \n"
unit_cell_constants = (
    "!UNIT_CELL_CONSTANTS=  59.22     45.66     86.77  90.000  91.275  90.000\n"
)
max_num_proc = "MAXIMUM_NUMBER_OF_PROCESSORS= 80  \n"
max_num_jobs = "MAXIMUM_NUMBER_OF_JOBS=       48  \n"
resolution_range = "INCLUDE_RESOLUTION_RANGE= 40 2.0 \n"

# reference_data_set = ''
reference_data_set = "REFERENCE_DATA_SET= %s \n" % "../reference.HKL"
if len(reference_data_set) > 0:
    use_reference = True
else:
    use_reference = False


fin = open(data_summary_table).read().split("\n")
log = open("log.express", "w")

# reading input files into XDSs
XDSs = dict()
for index, string in enumerate(fin):
    try:
        name, data, inp, data_range_start, data_range_stop = string.split(",")
    except ValueError:
        print("Error while loading string %d:\t%s" % (index, string), file=log)
        continue
    XDSs[name] = [data, inp, data_range_start, data_range_stop]

print("Following folders detected:\n", file=log)
for name in XDSs.keys():
    print("%s\n \t%s\n \t%s\n\n" % (name, XDSs[name][0], XDSs[name][1]), file=log)


# for each dataset folder does some stuff
for name in XDSs.keys():
    os.chdir(working_directory)

    xycorr = XDSs[name][1]
    xds = XDSs[name][1] + "/XDS.INP"
    data = XDSs[name][0]
    data_range_start = XDSs[name][2]
    data_range_stop = XDSs[name][3]

    os.chdir(name)
    project_dir = os.getcwd()
    # remember current directory

    copyfile("../XSCALE.express.py.INP", "XSCALE.INP")

    xds = open("XDS.INP", "r")
    modif = xds.readlines()
    xds.close()

    job = False
    spotrange_first = False
    for i, string in enumerate(modif):
        noreference = True

        if False:
            pass

        elif Find("SPACE_GROUP_NUMBER=", string):
            modif[i] = space_group
            print("### Space group added")

        elif Find("UNIT_CELL_CONSTANTS=", string):
            # modif[i] = 'UNIT_CELL_CONSTANTS= 36.337 35.631 41.277 90.000 93.606 90.000\n'
            modif[i] = unit_cell_constants
            print("### Unit cell constants added")

        elif Find("JOB=", string):
            if job:
                modif[i] = "\n"
            else:
                modif[i] = "JOB=XYCORR INIT COLSPOT IDXREF\n"
                job = True
            print("### Job added")

        elif Find("JOB=", string) and job == True:
            modif[i] = "\n"

        elif Find("SECONDS=", string):
            modif[i] = "!" + string
            print("### Seconds added")

        elif Find("MAXIMUM_NUMBER_OF_PROCESSORS=", string):
            modif[i] = max_num_proc
            print("### Maximum number of processors added")

        elif Find("MAXIMUM_NUMBER_OF_JOBS=", string):
            modif[i] = max_num_jobs
            print("### Maximum number of jobs added")

        elif Find("X-GEO_CORR=", string):
            os.system("bzip2 -d %s" % (xycorr + "/x_geo_corr.cbf.bz2\n"))
            # modif[i] = 'X-GEO_CORR=%s'%(xycorr + '/x_geo_corr.cbf\n')
            modif[i] = "images/x_geo_corr.cbf\n"
        #
        elif Find("Y-GEO_CORR=", string):
            os.system("bzip2 -d %s" % (xycorr + "/y_geo_corr.cbf.bz2\n"))
            # modif[i] = 'Y-GEO_CORR=%s'%(xycorr + '/y_geo_corr.cbf\n')
            modif[i] = "images/y_geo_corr.cbf\n"

        elif Find("RESOLUTION_RANGE", string):
            modif[i] = resolution_range
            print("### Resolution range added")

        elif Find("LIB", string):
            modif[
                i
            ] = "LIB=/home/marin/Apps/neggia/build/src/dectris/neggia/plugin/dectris-neggia.so \n"

        elif Find("REFERENCE_DATA_SET", string):
            modif[i] = ""
            if use_reference:
                modif[i] = reference_data_set
                noreference = False
            else:
                pass

        elif Find("SPOT_RANGE", string):
            modif[i] = "SPOT_RANGE= %s %s\n" % (data_range_start, data_range_stop)

    noreference = True
    if noreference:
        modif.append("REFERENCE_DATA_SET= ../reference.HKL\n")

    with open("XDS.INP", "w") as xds:
        xds.writelines(modif)

    xds.close()

    os.system("xds_par")
    os.system("xscale_par")
    os.system("cp XDS_ASCII.HKL XDS_ASCII.HKL_old")

    os.system("cp GXPARM.XDS XPARM.XDS")
    os.system("mv CORRECT.LP CORRECT.LP.old")
    os.system("mv XSCALE.LP XSCALE.LP.old")
    os.system("egrep -v 'JOB|REIDX' XDS.INP > XDS.INP.new")
    os.system(
        'echo "! JOB=XYCORR INIT COLSPOT IDXREF DEFPIX INTEGRATE CORRECT" > XDS.INP'
    )
    os.system('echo "JOB=DEFPIX INTEGRATE CORRECT" >> XDS.INP')
    os.system("cat XDS.INP.new >> XDS.INP")
    os.system("xds_par")
    os.system("xscale_par")


os.system("ls -alt")
log.close()
